function model=pls2pred(varargin)
%Applies a PLS model calculated by the function pls2mod to a new set of data
%for prediction. 
%I/O: pred=pls2mod(Xnew,Ynew, plsmodel)
%     pred=pls2mod(Xnew,plsmodel)
% depending on whether the Y values for the new set are available, or not
%

if nargin==3 
    Xt=varargin{1};
    Yt=varargin{2};
    modpls=varargin{3};
elseif nargin==2
    Xt=varargin{1};
    Yt=[];
    modpls=varargin{2};
    
end


%Preprocessing
%Apply to the unknown X the preprocessing parameters of the calibration set.
Xp=apprepfn(Xt,modpls.options.prepr{1},modpls.preprpars{1,1}, modpls.preprpars{1,2});
T=Xp*modpls.Rweights;
Yhat=Xp*modpls.regcoef; 
U=Yhat*modpls.loadings{2};
model.scores={T U}; 
model.loadings=modpls.loadings; 
Ypred=unprepfn(Yhat,modpls.options.prepr{2},modpls.preprpars{2,1}, modpls.preprpars{2,2});
model.preds=Ypred; 

if ~isempty(Yt)


    model.YResiduals=Yt-Ypred; 
    model.bias=mean(model.YResiduals); 
    model.RMSEP=sqrt(sum(model.YResiduals.^2)/size(Ypred,1)); 
    RSS=sum(model.YResiduals.^2);
    TSS=sum((Yt-repmat(mean(Yt), size(Yt,1), 1)).^2); 
    model.R2=1-(RSS./TSS);
    
end

if strcmp(modpls.options.detailedoutput,'on')
    
    q=sum((Xp-T*modpls.loadings{1}').^2,2);
    t2=diag((size(modpls.scores{1},1)-1)*T/(modpls.scores{1}'*modpls.scores{1})*T'); %tsquare
    
    
    
    model.tsq=t2;
    model.qres=q;
    model.tsqlim=modpls.tsqlim;
    model.qlim=modpls.qlim;
    model.tsqr=t2/modpls.tsqlim; 
    model.qr=q/modpls.qlim;
    
end










%%%%%%%%%%%%%%%%
function Mp=apprepfn(X,pret1,mm,sm)
%Preprocessing routine. Performs mean centering ('mean'), autoscaling
%('auto') or no pretreatment ('none') - Using parameters computed on the
%training set

nt=size(X,1);
switch pret1
    case 'none'
        Mp=X;
    case 'mean'
        Mp=X-repmat(mm, nt, 1);
    case {'auto','pareto'}
        snzero=1:size(X,2);snzero(sm==0)=[];
        Mp=zeros(size(X));
        Mp(:,snzero)=(X(:,snzero)-repmat(mm(snzero), nt, 1))./repmat(sm(snzero),nt, 1);
end


%%%%%%%%%%%%%%%%%%
function Mn=unprepfn(X,pret1,mm,sm)
%Preprocessing routine. Applies preprocessing to new matrices using the parameters
%calculated on the training set. Performs mean centering ('mean'), autoscaling
%('auto') or no pretreatment ('none')

nt=size(X,1);
switch pret1
    case 'none'
        Mn=X;
        
    case 'mean'
        Mn=X+repmat(mm, nt, 1);
    case {'auto', 'pareto'}
        Mn=(X.*repmat(sm,nt, 1))+repmat(mm, nt, 1);
end
