function model = plsdapred(varargin )
% Calculates PLS-DA predictions on new samples based on the model built
% either by plsdamod or plsdacv functions. 
%
% I/O:  pred=plsdapred(Xnew, plsdamodel) if no class information on new
%                                        samples is available, or:
% I/O:  pred=plsdapred(Xnew, Ynew, plsdamodel)samples is available.
%
% All the predictions are saved in the pred structure.



if nargin==3
    
    Xt=varargin{1};
    Yt=varargin{2};
    plsdamodel=varargin{3};
    
elseif nargin==2
    Xt=varargin{1};
    Yt=[];
    plsdamodel=varargin{2};
end

if ~isempty(Yt)
    if isvector(Yt)
        if size(Yt,1)==1
            Yt=Yt';
        end
        yclt=Yt;
        nclass=max(max(yclt), max(plsdamodel.classification.TrueClass));
        Yt=zeros(length(yclt), nclass);
    for i=1:length(yclt)
        Yt(i,yclt(i))=1;
    end
    Yt=Yt(:,1:max(plsdamodel.classification.TrueClass));
    else
        [~,yclt]=max(Yt,[],2);
    end
    
    
    
end

model=pls2pred(Xt,plsdamodel);

model.classification.TrueClass=[];
switch plsdamodel.options.classcriterion
    case 'maxY'
        [~,YPredClass]=max(model.preds,[],2);
        model.classification.PredClass=YPredClass;
    case 'lda'
        if strcmp(plsdamodel.options.ldax, 'scores')
            xx=plsdamodel.scores{1};
            xxt=model.scores{1};
        elseif strcmp(plsdamodel.options.ldax, 'predy')
            xx=plsdamodel.preds;
            xx=xx(:,1:end-1);
            xxt=model.preds;
            xxt=xxt(:,1:end-1);
            
        end
        
        mlda= ldaf(xx,xxt,plsdamodel.classification.TrueClass, plsdamodel.options.prior);
        model.classification.PredClass=mlda.PredClass;
        model.classification.probability=mlda.PosteriorProb;
end


if ~isempty(Yt)
    model.classification.TrueClass=yclt;
    cmatrix=cmatrixcalc(model.classification.PredClass, yclt, size(model.preds,2));
    model.classification.ConfusionMatrix=cmatrix;
    ccrate(1:size(cmatrix,2))=100*diag(cmatrix)./sum(cmatrix,2);
    model.classification.CorrClassRate=ccrate;
    model.classification.CorrClassTot=100*sum(diag(cmatrix))/length(yclt);
end


function cmatrix=cmatrixcalc(Ypred, Ytrue, nc)
ncts=max(Ytrue);

if ncts>nc
    cmatrix=zeros(ncts,nc);
else
    cmatrix=zeros(nc,nc);
end

for i=1:length(Ytrue)
    cmatrix(Ytrue(i), Ypred(i))=cmatrix(Ytrue(i), Ypred(i))+1;
end